#include "DHT.h"
#include <LiquidCrystal.h>

const int rs = 12, en = 11, d4 = 5, d5 = 4, d6 = 3, d7 = 2;
LiquidCrystal lcd(rs, en, d4, d5, d6, d7);

int maxPeople; // Initial maximum number of people allowed before the alarm goes off
int sensitivity = 2; //lower values will make it more sensitive and higher values will make it less sensitive
int state = 0;
int pressed = 0;
int lastPressed = 0;
int invert = 0;
int invertPressed = 0;
int wait = 0;
int waitState = 0;
int i = 0;
//---------------------------------------------------
// DHT pin
#define DHTPIN 10
// Sensor Type
#define DHTTYPE DHT11
// Initialize DHT11 sensor
DHT dht(DHTPIN, DHTTYPE);
// Variables and pins declaration
int currentPeople = 0;

int buzzer = A0;
int ledVerde = A1;
int ledRojo = A2;
int button1 = 13;

int sensor1[] = {8, 9};
int sensor2[] = {6, 7};
int sensor1Initial;
int sensor2Initial;

String sequence = "";

int timeoutCounter = 0;

void setup() {
  //Setup code
  Serial.begin(9600);
  pinMode(buzzer, OUTPUT);
  pinMode(button1, INPUT);
  lcd.begin(16, 2);

  lcd.setCursor(0, 0);
  lcd.print("Welcome");
  lcd.setCursor(1, 1);
  lcd.print("By: Paolo Reyes");
  sensor1Initial = measureDistance(sensor1);
  sensor2Initial = measureDistance(sensor2);
  Serial.println(sensor1Initial);
  Serial.println(sensor2Initial);
  dht.begin();

  delay(2500);
  lcd.clear();
}

void loop() {

  pressed = digitalRead(13);
  invertPressed = analogRead(A4);
  wait = analogRead(A5);

  if (wait > 500) {
    if (waitState == 0) {
      waitState++;
      delay(1000);
      Serial.print(waitState);
    } else if (waitState == 1) {
      waitState--;
      i--;
      delay(1000);
      Serial.print(waitState);
    }
  }

  if (waitState == 0) {
    if (invertPressed > 500) {
      if (invert == 0) {
        invert++;
        delay(1000);
      } else if (invert == 1) {
        invert--;
        delay(1000);
      }
    }

    if (pressed == HIGH) {
      if (state == 0) {
        state++;
        delay(1000);
      } else if (state == 1) {
        state--;
        delay(1000);
      }
    }
    Serial.print (" Estado: ");
    Serial.print (state);

    maxPeople = analogRead(A3) / 25;
    Serial.print(" Max People: ");
    Serial.print(maxPeople);

    int sensor1Val = 0;
    int sensor2Val = 0;

    //Read ultrasonic sensors
    if (invert == 0) {
      lcd.setCursor(15, 1);
      lcd.print("N");
      sensor1Val = measureDistance(sensor1);
      sensor2Val = measureDistance(sensor2);
      Serial.print(" normal ");
      lcd.setCursor(15, 1);
      lcd.print("N");
    } else if (invert == 1) {
      lcd.setCursor(15, 1);
      lcd.print("I");
      sensor1Val = measureDistance(sensor2);
      sensor2Val = measureDistance(sensor1);
      Serial.print(" inverted ");
      lcd.setCursor(15, 1);
      lcd.print("I");
    }

    //Process the data
    if (sensor1Val < sensor1Initial - 10 && sequence.charAt(0) != '1') {
      sequence += "1";
    } else if (sensor2Val < sensor2Initial - 10 && sequence.charAt(0) != '2') {
      sequence += "2";
    }

    if (sequence.equals("12")) {
      currentPeople++;
      sequence = "";
      delay(550);
    } else if (sequence.equals("21") && currentPeople > 0) {
      currentPeople--;
      sequence = "";
      delay(550);
    }

    //Resets the sequence if it is invalid or timeouts
    if (sequence.length() > 2 || sequence.equals("11") || sequence.equals("22") || timeoutCounter > 10) {
      sequence = "";
    }

    if (sequence.length() == 1) { //
      timeoutCounter++;
    } else {
      timeoutCounter = 0;
    }

    //Print values to serial
    Serial.print(" Seq: ");
    Serial.print(sequence);
    Serial.print(" S1: ");
    Serial.print(sensor1Val);
    Serial.print(" S2: ");
    Serial.println(sensor2Val);
    Serial.print("Current People: ");
    Serial.print(currentPeople);

    //If the number of people is too high, trigger the buzzer
    if (currentPeople > maxPeople) {
      analogWrite (buzzer, 1023);
    } else {
      analogWrite (buzzer, 0);
    }

    if (currentPeople <= maxPeople) {
      analogWrite (ledVerde, 1023);
    } else {
      analogWrite (ledVerde, 0);
    }

    if (currentPeople > maxPeople) {
      analogWrite (ledRojo, 1023);
    } else {
      analogWrite (ledRojo, 0);
    }

    int h = dht.readHumidity();
    int t = dht.readTemperature();

    if (isnan (h) || isnan (t)) {
      Serial.println("Error getting data");
      return;
    }

    float hic = dht.computeHeatIndex(t, h, false);

    Serial.print(" Humedad: ");
    Serial.print(h);
    Serial.print(" %\t");
    Serial.print(" Temperatura ");
    Serial.print(t);
    Serial.print(" °C ");

    if (state == 0) {
      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("People Inside:");
      lcd.print(currentPeople);
      lcd.setCursor(0, 1);
      lcd.print("Max People:");
      lcd.print(maxPeople);
    } else if (state == 1) {
      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("Temperature: ");
      lcd.print(t);
      lcd.print("C");
      lcd.setCursor(0, 1);
      lcd.print("Humidity: ");
      lcd.print(h);
      lcd.print("%");
    }
  } else if (waitState == 1) {
    if (i == 0) {
      lcd.clear();
      i++;
    } else if (i == 1) {
      lcd.setCursor(0, 0);
      lcd.print("Program stopped");
      analogWrite (ledVerde, 0);
      analogWrite (ledRojo, 1023);
    }
  }
}

//Returns the distance of the ultrasonic sensor that is passed in
//a[0] = echo, a[1] = trig
int measureDistance(int a[]) {
  pinMode(a[1], OUTPUT);
  digitalWrite(a[1], LOW);
  delayMicroseconds(2);
  digitalWrite(a[1], HIGH);
  delayMicroseconds(10);
  digitalWrite(a[1], LOW);
  pinMode(a[0], INPUT);
  long duration = pulseIn(a[0], HIGH, 100000);
  return duration / 29 / 2;
}
